/***************************************************************************
                          z80emulaterdoc.cpp  -  description
                             -------------------
    begin                :  10 16 20:27:21 JST 2001
    copyright            : (C) 2001 by zakki
    email                : zakki@peppermint.jp
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "z80emulaterdoc.h"
#include <qfile.h>
#include <qregexp.h>
#include <qtextstream.h>

#include <iostream>
#include <fstream>
#include <stdio.h>
#include <curses.h>
#include <unistd.h>
#include <termios.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <sys/times.h>

static Z80emulaterDoc* s_doc = NULL;

Z80emulaterDoc::Z80emulaterDoc()
{
	modified = false;
	s_doc = this;
}

Z80emulaterDoc::~Z80emulaterDoc()
{
}

void Z80emulaterDoc::newDoc()
{
}

bool Z80emulaterDoc::save()
{
	return true;
}

bool Z80emulaterDoc::saveAs(const QString &filename)
{
	return true;
}

bool Z80emulaterDoc::load(const QString &filename)
{
	QFile file(filename);
	file.open(IO_ReadOnly);
	QTextStream stream(&file);
	
	QString line;
	
	QRegExp codeLineExp("^[0-9A-Fa-f]+: [0-9A-Fa-f]+");
	QRegExp addrExp("^[0-9A-Fa-f]+");
	QRegExp codeExp("[0-9A-Fa-f]+");

	int lineNum = 0;
	while(stream.atEnd()==0){
		line = stream.readLine();
		m_codes.push_back(line);
				
		if(codeLineExp.find(line, 0) != -1){
			int addrLen = 0;
			int addrStart = addrExp.match(line, 0, &addrLen);
			int codeLen = 0;
			int codeStart = codeExp.match(line, addrStart+addrLen, &codeLen);
			
			if(codeLen%2 != 0){continue;}
			
			QString addrStr = line.mid(addrStart, addrLen);
			QString codeStr = line.mid(codeStart, codeLen);
			int addr = addrStr.toInt(0, 16);
			cout << addr << ":";
			m_addrToLine[addr] = lineNum;
			for(int i = 0; i < codeStr.length()/2; ++ i, ++addr){
				byte mem = codeStr.mid(i*2, 2).toInt(0, 16);
				m_mem[addr] = mem;
				cout << (int)mem;
			}
			cout << endl;
		}
		++ lineNum;
	}
	cout << "loaded" << endl;
	
	slotReset();
	
	emit documentChanged();
	return true;
}

int Z80emulaterDoc::getCurrentLineNo()
{
	return m_addrToLine[m_z80.PC.W];
}
QString Z80emulaterDoc::getCurrentLine()
{
	return m_codes[getCurrentLineNo()];
}
vector<QString>& Z80emulaterDoc::getCodes()
{
	return m_codes;
}
void Z80emulaterDoc::timerEvent(QTimerEvent* te)
{
	for(int i = 0; i < m_updateCycle; ++ i){
		ExecZ80(&m_z80);
	}
	emit documentChanged();
}

void Z80emulaterDoc::slotReset()
{
	//cout << "Reset." << endl;
	ResetZ80(&m_z80);
	
	m_z80.PC.W = 0x8000;
	m_z80.Trace = 0;
}
void Z80emulaterDoc::slotRun(int cycle)
{
	m_updateCycle = cycle;
	//cout << "Run." << endl;
	startTimer(1);
}
void Z80emulaterDoc::slotStep()
{
	ExecZ80(&m_z80);
	emit documentChanged();
}
void Z80emulaterDoc::slotStop()
{
	//cout << "Stop." << endl;
	killTimers();
}
bool Z80emulaterDoc::isModified() const
{
	return modified;
}
Z80* Z80emulaterDoc::getZ80()
{
	return &m_z80;
}
void Z80emulaterDoc::WrZ80(register word Addr,register byte Value)
{
	//printf("Wr %Xh:%Xh\n", Addr, g_mem[Addr]);
	m_mem[Addr] = Value;
}
byte Z80emulaterDoc::RdZ80(register word Addr)
{
	//printf("Rd %Xh:%Xh\n", Addr, g_mem[Addr]);
	return m_mem[Addr];
}
void Z80emulaterDoc::OutZ80(register word Port,register byte Value)
{
	if(Port==0x18){
		printf("%c", Value);
	}else{
		cerr << "Unknown OUT Port." << endl;
	}
	fflush(stdout);
}
byte Z80emulaterDoc::InZ80(register word Port)
{
	switch(Port){
	case 0x18:
		{ //# ʸ
			char c;
			//read(STDIN, c, 1);
			//cin >> c;
			//cout << "+in 18" << endl;
			/*
			int bytes;
			if(ioctl(0, FIONREAD, &bytes)==-1){
				cout << errno << endl;
				exit(0);
			}else if(bytes==0){
				c = 0;
			}else{
				c = getchar();
				//read(0, &c, 1);
			}*/
			fd_set rfds;
			timeval tv;
			FD_ZERO(&rfds);
			FD_SET(0, &rfds);
			tv.tv_sec = 0;
			tv.tv_usec = 0;
			
			int retval = select(1, &rfds, NULL, NULL, &tv);
			if(retval){
				//cout << "++in 18" << endl;
				c = getchar();
				//cout << "--in 18" << endl;
			}else{
				c = 0;
			}
			//read(1, &c, 1);
			//cout << "-in 18" << endl;
			return c;
		}
		break;
	case 0x19:
		{
			int status = 4; //# ϤϤĤǤǽȤ
            /*//# ϲǽå
			int read_bits = 0;
			vec(read_bits, fileno(STDIN), 1) = 1;
			select(read_bits, undef, undef, 0);
			if ( vec(read_bits, fileno(STDIN), 1) ) {
				status |= 1;
			}*/
			fd_set rfds;
			timeval tv;
			FD_ZERO(&rfds);
			FD_SET(0, &rfds);
			tv.tv_sec = 0;
			tv.tv_usec = 0;
			
			int retval = select(1, &rfds, NULL, NULL, &tv);
			if(retval){
				status |= 1;
				//cout << "t" << endl;
			}else{
				//cout << "f" << endl;
			}
			//cout << "+in 19" << endl;
			/*int bytes;
			if(ioctl(0, FIONREAD, &bytes)==-1){
				cout << errno << endl;
				exit(0);
			}else if(bytes==0){
			}else{
				status |= 1;
				//cout << "read:" << getchar() << endl;
			}*/
			//cout << "-in 19" << endl;
			return status;
		}
		break;
	default:
		cerr << "Unknown IN Port." << endl;
		return 0;
		break;
	}
}
void Z80emulaterDoc::PatchZ80(register Z80 *R)
{
}
word Z80emulaterDoc::LoopZ80(register Z80 *R)
{
	//called per 10k clock.
	cout << "LoopZ80 " << R->ICount << "/" << R->IPeriod <<  endl;
	//R->ICount = 0;
	//R->IPeriod = 1000;
/*	struct timeval tv;
	tv.tv_sec = 0;
	tv.tv_usec = 1000;
	select(0, NULL, NULL, NULL, &tv);*/
	return INT_NONE;
}

void WrZ80(register word Addr,register byte Value)
{
	s_doc->WrZ80(Addr, Value);
}
byte RdZ80(register word Addr)
{
	return s_doc->RdZ80(Addr);
}
void OutZ80(register word Port,register byte Value)
{
	s_doc->OutZ80(Port, Value);
}
byte InZ80(register word Port)
{
	return s_doc->InZ80(Port);
}
void PatchZ80(register Z80 *R)
{
	s_doc->PatchZ80(R);
}
word LoopZ80(register Z80 *R)
{
	return s_doc->LoopZ80(R);
}

