/***************************************************************************
                          z80emulaterview.cpp  -  description
                             -------------------
    begin                :  10 16 20:27:21 JST 2001
    copyright            : (C) 2001 by zakki
    email                : zakki@peppermint.jp
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "z80emulaterview.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qvalidator.h>

Z80emulaterView::Z80emulaterView(QWidget *parent, Z80emulaterDoc *doc) : QWidget(parent)
{
	/** connect doc with the view*/
	connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
	m_doc = doc;
	
	QHBoxLayout* topLayout = new QHBoxLayout(this);
	{
		m_asmCode = new QMultiLineEdit(this);
		m_asmCode->setReadOnly(true);
		m_asmCode->setMinimumWidth(400);
		topLayout->addWidget(m_asmCode);
		
		QVBoxLayout* edit = new QVBoxLayout(topLayout);
		{
			QGridLayout* registers = new QGridLayout(edit, 0, 0, 10);
			QLabel* afLabel = new QLabel(this);
			afLabel->setText("AF");
			registers->addWidget(afLabel, 0, 0);
			m_AF = new QLineEdit(this);
			registers->addWidget(m_AF, 0, 1);
			
			QLabel* bcLabel = new QLabel(this);
			bcLabel->setText("BC");
			registers->addWidget(bcLabel, 1, 0);
			m_BC = new QLineEdit(this);
			registers->addWidget(m_BC, 1, 1);
			
			QLabel* deLabel = new QLabel(this);
			deLabel->setText("DE");
			registers->addWidget(deLabel, 2, 0);
			m_DE = new QLineEdit(this);
			registers->addWidget(m_DE, 2, 1);
			
			QLabel* hlLabel = new QLabel(this);
			hlLabel->setText("HL");
			registers->addWidget(hlLabel, 3, 0);
			m_HL = new QLineEdit(this);
			registers->addWidget(m_HL, 3, 1);
			
			QLabel* ixLabel = new QLabel(this);
			ixLabel->setText("IX");
			registers->addWidget(ixLabel, 4, 0);
			m_IX = new QLineEdit(this);
			registers->addWidget(m_IX, 4, 1);
			
			QLabel* iyLabel = new QLabel(this);
			iyLabel->setText("IY");
			registers->addWidget(iyLabel, 5, 0);
			m_IY = new QLineEdit(this);
			registers->addWidget(m_IY, 5, 1);
			
			QLabel* pcLabel = new QLabel(this);
			pcLabel->setText("PC");
			registers->addWidget(pcLabel, 6, 0);
			m_PC = new QLineEdit(this);
			registers->addWidget(m_PC, 6, 1);
			
			QLabel* spLabel = new QLabel(this);
			spLabel->setText("SP");
			registers->addWidget(spLabel, 7, 0);
			m_SP = new QLineEdit(this);
			registers->addWidget(m_SP, 7, 1);
			
			QLabel* cyLabel = new QLabel(this);
			cyLabel->setText("Cycle");
			registers->addWidget(cyLabel, 8, 0);
			m_CYCLE = new QLineEdit(this);
			registers->addWidget(m_CYCLE, 8, 1);
			
			QLabel* af1Label = new QLabel(this);
			af1Label->setText("AF1");
			registers->addWidget(af1Label, 0, 2);
			m_AF1 = new QLineEdit(this);
			registers->addWidget(m_AF1, 0, 3);
			
			QLabel* bc1Label = new QLabel(this);
			bc1Label->setText("BC1");
			registers->addWidget(bc1Label, 1, 2);
			m_BC1 = new QLineEdit(this);
			registers->addWidget(m_BC1, 1, 3);
			
			QLabel* de1Label = new QLabel(this);
			de1Label->setText("DE1");
			registers->addWidget(de1Label, 2, 2);
			m_DE1 = new QLineEdit(this);
			registers->addWidget(m_DE1, 2, 3);
			
			QLabel* hl1Label = new QLabel(this);
			hl1Label->setText("HL1");
			registers->addWidget(hl1Label, 3, 2);
			m_HL1 = new QLineEdit(this);
			registers->addWidget(m_HL1, 3, 3);
			
			QLabel* iffLabel = new QLabel(this);
			iffLabel->setText("IFF");
			registers->addWidget(iffLabel, 5, 2);
			m_IFF = new QLineEdit(this);
			registers->addWidget(m_IFF, 5, 3);
			
			QLabel* iLabel = new QLabel(this);
			iLabel->setText("I");
			registers->addWidget(iLabel, 6, 2);
			m_I = new QLineEdit(this);
			registers->addWidget(m_I, 6, 3);
			
			QHBoxLayout* debugPanel = new QHBoxLayout(edit);
			{
				m_trace = new QCheckBox(this, "Do trace.");
				debugPanel->addWidget(m_trace);
				
				QLabel* upLabel = new QLabel(this);
				upLabel->setText("Update cycles");
				debugPanel->addWidget(upLabel);
				m_updateCount = new QLineEdit(this);
				debugPanel->addWidget(m_updateCount);
				QIntValidator* validator = new QIntValidator(this);
				validator->setBottom(1);
				m_updateCount->setValidator(validator);
				m_updateCount->setText("1");
			}
			
			QVBoxLayout* buttons = new QVBoxLayout(edit);
			{
				m_run = new QPushButton(this);
				m_run->setText("Run");
				buttons->addWidget(m_run);
				
				m_step = new QPushButton(this);
				m_step->setText("Step");
				buttons->addWidget(m_step);
				
				m_stop = new QPushButton(this);
				m_stop->setText("Stop");
				buttons->addWidget(m_stop);
				
				m_reset = new QPushButton(this);
				m_reset->setText("Reset");
				buttons->addWidget(m_reset);
			}
		}
	}
	connect(m_run, SIGNAL(clicked()), this, SLOT(slotRun()));
	connect(m_step, SIGNAL(clicked()), this, SLOT(slotStep()));
	connect(m_stop, SIGNAL(clicked()), this, SLOT(slotStop()));
	connect(m_reset, SIGNAL(clicked()), this, SLOT(slotReset()));
}
void Z80emulaterView::slotRun()
{
	m_doc->slotRun(m_updateCount->text().toInt());
}
void Z80emulaterView::slotStep()
{
	m_doc->slotStep();
}
void Z80emulaterView::slotStop()
{
	m_doc->slotStop();
}
void Z80emulaterView::slotReset()
{
	m_doc->slotReset();
}
Z80emulaterView::~Z80emulaterView()
{
	
}

void Z80emulaterView::slotDocumentChanged()
{
	//TODO update the view
	//cout << "void Z80emulaterView::slotDocumentChanged() start" << endl;
	if(m_asmCode->numLines() != m_doc->getCodes().size()){
		cout << "Update Asm Code." << m_asmCode->numLines() << ","
			<< m_doc->getCodes().size()<< endl;
		m_asmCode->clear();
		vector<QString>& code = m_doc->getCodes();
		for(int i = 0; i < code.size(); ++ i){
			m_asmCode->insertLine(code[i]);
		}
	}
	QString num;
	m_AF->setText(num.setNum(m_doc->getZ80()->AF.W, 16));
	m_AF1->setText(num.setNum(m_doc->getZ80()->AF1.W, 16));
	m_BC->setText(num.setNum(m_doc->getZ80()->BC.W, 16));
	m_BC1->setText(num.setNum(m_doc->getZ80()->BC1.W, 16));
	m_DE->setText(num.setNum(m_doc->getZ80()->DE.W, 16));
	m_DE1->setText(num.setNum(m_doc->getZ80()->DE1.W, 16));
	m_HL->setText(num.setNum(m_doc->getZ80()->HL.W, 16));
	m_HL1->setText(num.setNum(m_doc->getZ80()->HL1.W, 16));
	m_IX->setText(num.setNum(m_doc->getZ80()->IX.W, 16));
	m_IY->setText(num.setNum(m_doc->getZ80()->IY.W, 16));
	m_PC->setText(num.setNum(m_doc->getZ80()->PC.W, 16));
	m_SP->setText(num.setNum(m_doc->getZ80()->SP.W, 16));
	m_IFF->setText(num.setNum(m_doc->getZ80()->IFF, 16));
	m_I->setText(num.setNum(m_doc->getZ80()->I, 16));
	
	if(m_trace->isChecked()){
		m_asmCode->deselect();
		int currentLine = m_doc->getCurrentLineNo();
		m_asmCode->setCursorPosition (currentLine, 0, false);
		m_asmCode->setCursorPosition (currentLine + 1, 0, true);
	}
	//cout << "void Z80emulaterView::slotDocumentChanged() ok." << endl;
}
